
#include <SPI.h>
#include <SD.h>
#include <time.h>
#include <String.h>

#include "panic.h"

#include "stopwatch.h"

#include "Experiment.h"

#include "sched_time.h"
#include "tasks.h"
#include "schedule.h"

void setup()
{
	// use pin to relay status information
	pinMode(LED_BUILTIN, OUTPUT);
	digitalWrite(LED_BUILTIN, LOW);

	// initialize arduino and the debug environment:
	Serial.begin(9600);
	while (!Serial) {
		digitalWrite(LED_BUILTIN, HIGH);
		delay(100);
		digitalWrite(LED_BUILTIN, LOW);
		delay(100);
	}

    Serial.print(F("***************************** "));
    Serial.print(F(SCHEDULER_NAME));
    Serial.println(F(" *****************************"));

	bool sd_ok;
	sd_ok = SD.begin(4);
	while (!sd_ok) {
		Serial.println("Retrying SD");
		digitalWrite(LED_BUILTIN, HIGH);
		delay(1000);
		digitalWrite(LED_BUILTIN, LOW);
		delay(750);
		sd_ok = SD.begin(4);
	}

	BUG_ON(!SD.exists("RESULTS"));

	digitalWrite(LED_BUILTIN, HIGH);
#ifdef TABLES_IN_FLASH
	Serial.println(F("One-shot mode: starting experiment..."));
#else
	Serial.println(F("Looking for unfinished experiments..."));
#endif
	start_next_experiment();
}

static void experiment_tick()
{
	if (experiment_has_finished()) {
		end_experiment();
#ifdef TABLES_IN_FLASH
		// single-shot mode --- we are done, just terminate
		BUG_ON("finished");
#else
		// multi-experiment mode --- start the next one
		start_next_experiment();
#endif
	}
}

StopWatch overhead;

void loop()
{
	overhead.start();
	/* query the scheduler if we should do something */
	sched_decision_t decision = schedule();

	/* if the scheduler found some work, let's do it */
	if (!decision.is_idle()) {
		overhead.stop();
		decision.dispatch();
	}
	experiment_tick();
}
