#include <SPI.h>
#include <SD.h>
#include "Arduino.h"

#include "panic.h"

#include "schedule.h"

bool cat(String fname)
{
	File file = SD.open(fname);
	char c;

	if (!file) {
		bool exists = SD.exists(fname);
		Serial.print(fname);
		Serial.print(": could not open --- exists -> ");
		Serial.println(exists);
		return false;
	}

	Serial.println(fname + ':');
	Serial.println("----");
	do {
		c = file.read();
		if (c != -1 && c != '\r' && c != '\n')
			Serial.print(c);
		if (c == '\n')
			Serial.println();
	} while (c != -1);
	Serial.println("----");
	file.close();
	return true;
}

unsigned long count_lines(File &file)
{
	unsigned long count = 0;
	char c;

	do {
		c = file.read();
		if (c == '\n')
			count++;
	} while (c != -1);

	return count;
}

void skip_lines(File &file, byte how_many)
{
	char c = 0;

	while (how_many && c != -1) {
		c = file.read();
		if (c == '\n')
			how_many--;
	}
}

bool parse_value(File &file, unsigned long &val, char stop_char)
{
	String line = file.readStringUntil(stop_char);
	if (!line)
		return false;
	val = (unsigned long) line.toInt();
	return true;
}

bool parse_2tuple(File &file, unsigned long &val1, unsigned long &val2)
{
	return parse_value(file, val1, ' ') && parse_value(file, val2, '\n');
}

bool parse_3tuple(File &file,
	unsigned long &val1, unsigned long &val2, unsigned long &val3)
{
	return parse_value(file, val1, ' ')
		&& parse_value(file, val2, ' ')
		&& parse_value(file, val3, '\n');
}

void note(const char* msg)
{
	File notes_file = SD.open("notes.txt", FILE_WRITE);
	BUG_ON(!notes_file);
	notes_file.print(msg);
	notes_file.close();
}

void note(const __FlashStringHelper * msg)
{
	File notes_file = SD.open("notes.txt", FILE_WRITE);
	BUG_ON(!notes_file);
	notes_file.print(msg);
	notes_file.close();
}

void note(const String &msg)
{
	note(msg.c_str());
}
