/* An implementation of non-preemptive, work-conserving
 * periodic fixed-priority scheduling. */

#include "Arduino.h"

#include "tasks.h"
#include "sched_time.h"

static unsigned long next_release_time[MAX_TASKS];

void fp_init(void)
{
	for (int i = 0; i < MAX_TASKS; i++)
		next_release_time[i] = 0UL;
}

static void fp_release_job(byte i)
{
	/* unsigned long: possible overflow accepted and ok */
	next_release_time[i] += tasks[i].period;
}

sched_decision_t fp_schedule(void)
{
	unsigned long now = current_sched_time();

	/* ASSUMPTION: task index zero is the highest-priority task */
	for (byte i = 0; i < number_of_tasks; i++)
	{
		if (time_after(now, next_release_time[i])) {
			/* a job of this task has been released */
			fp_release_job(i);
			return (sched_decision_t)
				{tasks[i].job_function, tasks[i].wcet};
		}
	}

	/* If we reach here, no job is ready. Just idle. */
	return NO_JOB_READY;
}
